unit Main;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs, OpenGl,
  StdCtrls, ExtCtrls, ComCtrls, Menus, paramParser;

const PE_SCALE = 0;
      PE_ROTATE = 1;
      PE_TRANSLATE = 2;

type oglMatrix = array[0..15] of Single;

type TPrimitive = (Cube, Triangle, Square);

type

  TMatrixProtocol = class
    MatrixArray : array of oglMatrix;
    Length : integer;
    Index : integer;
    constructor Create;
    procedure add(mat : oglMatrix);
    function activate(i : integer) : boolean;
    procedure Clear;
  end;

  TfrmMain = class(TForm)
    oglScreen: TPanel;
    gbMatrix: TGroupBox;
    MainMenu1: TMainMenu;
    OpenGl1: TMenuItem;
    Init1: TMenuItem;
    Destroy1: TMenuItem;
    Render1: TMenuItem;
    N1: TMenuItem;
    Close1: TMenuItem;
    edma00: TEdit;
    edma10: TEdit;
    btnApplyMatrix: TButton;
    edma20: TEdit;
    edma30: TEdit;
    edma01: TEdit;
    edma11: TEdit;
    edma21: TEdit;
    edma31: TEdit;
    edma03: TEdit;
    edma32: TEdit;
    edma22: TEdit;
    edma13: TEdit;
    edma23: TEdit;
    edma33: TEdit;
    edma02: TEdit;
    edma12: TEdit;
    btnUndo: TButton;
    P1: TMenuItem;
    Triangle1: TMenuItem;
    Cube1: TMenuItem;
    Info1: TMenuItem;
    About1: TMenuItem;
    gbCommands: TGroupBox;
    Label1: TLabel;
    btnLoadIdentity: TButton;
    btnTranslate: TButton;
    btnRotate: TButton;
    btnScale: TButton;
    Square1: TMenuItem;
    gbProtocol: TGroupBox;
    cbMatrices: TComboBox;
    btnClearProtocol: TButton;
    btnBack: TButton;
    btnForward: TButton;
    cbParam: TComboBox;
    procedure Init1Click(Sender: TObject);
    procedure Destroy1Click(Sender: TObject);
    procedure Close1Click(Sender: TObject);
    procedure Render1Click(Sender: TObject);
    procedure btnUndoClick(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure FormClose(Sender: TObject; var Action: TCloseAction);
    procedure About1Click(Sender: TObject);
    procedure btnApplyMatrixClick(Sender: TObject);
    procedure btnTranslateClick(Sender: TObject);
    procedure btnLoadIdentityClick(Sender: TObject);
    procedure btnScaleClick(Sender: TObject);
    procedure btnRotateClick(Sender: TObject);
    procedure Triangle1Click(Sender: TObject);
    procedure Cube1Click(Sender: TObject);
    procedure Square1Click(Sender: TObject);
    procedure btnClearProtocolClick(Sender: TObject);
    procedure cbMatricesChange(Sender: TObject);
    procedure btnBackClick(Sender: TObject);
    procedure btnForwardClick(Sender: TObject);
  private
    { Private-Deklarationen }
    glContext : HGLRC;
    glDC : HDC;
    openGLReady: boolean;
    matrix : oglMatrix;
    Primitive : TPrimitive;
    MatrixProtocol : TMatrixProtocol;
    ParserOut : string;
    procedure printMatrix;
    procedure ProtocolClearAbove;
    procedure ParamError(ParamType : byte);
  public
    { Public-Deklarationen }
  end;

var
  frmMain: TfrmMain;

implementation

{$R *.DFM}

constructor TMatrixProtocol.Create;
begin
  Length := 1;
  setLength(MatrixArray,Length);
  Index := 0;
end;

procedure TMatrixProtocol.add(mat : oglMatrix);
begin
  Inc(Index);
  Length := Index+1;
  setLength(MatrixArray,Length);
  MatrixArray[Index] := mat;
end;

function TMatrixProtocol.activate(i : integer) : boolean;
begin
 if i+1 < Length then begin
   Index := i+1;
   glLoadMatrixf(@MatrixArray[Index]);
 end;
end;

procedure TMatrixProtocol.Clear;
begin
  Length := 1;
  setLength(MatrixArray,Length);
  Index := 0;
end;

procedure TfrmMain.ProtocolClearAbove;
var i, loops : integer;
begin
  if cbMatrices.Items.Count > MatrixProtocol.Index then begin
    loops := cbMatrices.Items.Count - MatrixProtocol.Index;
    for i := 1 to loops do cbMatrices.Items.Delete(MatrixProtocol.Index);
    cbMatrices.ItemIndex:=MatrixProtocol.Index;
  end;

end;

procedure TfrmMain.printMatrix;
begin
  edma00.text := floattostr(Matrix[0]);
  edma10.text := floattostr(Matrix[1]);
  edma20.text := floattostr(Matrix[2]);
  edma30.text := floattostr(Matrix[3]);
  edma01.text := floattostr(Matrix[4]);
  edma11.text := floattostr(Matrix[5]);
  edma21.text := floattostr(Matrix[6]);
  edma31.text := floattostr(Matrix[7]);
  edma02.text := floattostr(Matrix[8]);
  edma12.text := floattostr(Matrix[9]);
  edma22.text := floattostr(Matrix[10]);
  edma32.text := floattostr(Matrix[11]);
  edma03.text := floattostr(Matrix[12]);
  edma13.text := floattostr(Matrix[13]);
  edma23.text := floattostr(Matrix[14]);
  edma33.text := floattostr(Matrix[15]);
end;

procedure TfrmMain.ParamError(ParamType : byte);
begin
  if ParamType = PE_ROTATE then showmessage('Could not parse command parameters for glRotate...'+#13+#13+parserLog);
  if ParamType = PE_TRANSLATE then showmessage('Could not parse command parameters for glTranslate...'+#13+#13+parserLog);
  if ParamType = PE_SCALE then showmessage('Could not parse command parameters for glScale..'+#13+#13+parserLog);
end;


procedure TfrmMain.Init1Click(Sender: TObject);
var FormatIndex : integer;
    pfd : TPixelFormatDescriptor;
begin;

  MatrixProtocol := TMatrixProtocol.Create;

  if not OpenGLready then begin
    //Application.OnException := ExceptionGL;
    fillchar(pfd,SizeOf(pfd),0);
    with pfd do begin
        nSize      := SizeOf(pfd);
        nVersion   := 1; {The current version of the desccriptor is 1}
        dwFlags    := PFD_DRAW_TO_WINDOW or PFD_SUPPORT_OPENGL  or PFD_DOUBLEBUFFER;
        iPixelType := PFD_TYPE_RGBA;
        cColorBits := 24; {support 24-bit color}
        cDepthBits := 32; {depth of z-axis}
        iLayerType := PFD_MAIN_PLANE;
    end; {with}

    glDC := getDC(oglScreen.handle);
    FormatIndex := ChoosePixelFormat(glDC,@pfd);
    if FormatIndex=0 then
     raise Exception.Create('ChoosePixelFormat failed ' + IntToStr(GetLastError));

    if not SetPixelFormat(glDC,FormatIndex,@pfd) then
     raise Exception.Create('SetPixelFormat failed ' + IntToStr(GetLastError));

    GLContext := wglCreateContext(glDC);
    if GLContext=0 then
     raise Exception.Create('wglCreateContext failed ' + IntToStr(GetLastError));

    if not wglMakeCurrent(glDC,GLContext) then
     raise Exception.Create('wglMakeCurrent failed '+ IntToStr(GetLastError));

    glViewport(0, 0, oglScreen.Width, oglScreen.Height);    // Setzt den Viewport fr das OpenGL Fenster
    glMatrixMode(GL_PROJECTION);        // Matrix Mode auf Projection setzen
    glLoadIdentity();                   // Reset View
    gluPerspective(90.0, oglScreen.Width/oglScreen.Height, 1.0, 100.0);  // Perspektive den neuen Maen anpassen.
    // Perspektive den neuen Maen anpassen.
    glMatrixMode(GL_MODELVIEW);         // Zurck zur Modelview Matrix
    glLoadIdentity();
    glEnable(GL_TEXTURE_2D);	       // Aktiviert Texture Mapping
    glClearColor(0.0, 0.0, 0.0, 0.5);    // Bildschirm lschen (schwarz)
    glClearDepth(1.0);		       // Depth Buffer Setup
    glEnable(GL_DEPTH_TEST);	       // Aktiviert Depth Testing
    glDepthFunc(GL_LEQUAL);	       // Bestimmt den Typ des Depth Testing

    glTranslate(0,0,-5); //setting ModelViewMatrix to standard
    //die aktuelle Matrix speichern
    glGetFloatv(GL_MODELVIEW_MATRIX, @Matrix);
    //und ausgeben
    printMatrix;

    cbMatrices.Items.Add('Initial Matrix');
    cbMatrices.ItemIndex := MatrixProtocol.Index;
    MatrixProtocol.add(Matrix);

    OpenGLReady := true;
    oglScreen.Caption := 'OpenGL is ready to render!';

  end;
end;

procedure TfrmMain.Destroy1Click(Sender: TObject);
begin
 if OpenGlready then begin
   wglMakeCurrent(glDC, 0);

   wglDeleteContext(glContext);

   ReleaseDC(oglScreen.handle, glDC);

   OpenGlready := false;

   oglScreen.Caption := 'OpenGL is not initialized!'
 end;

 MatrixProtocol.Free;
end;

procedure TfrmMain.Close1Click(Sender: TObject);
begin
 Close;
end;

procedure TfrmMain.Render1Click(Sender: TObject);
begin
  if OpenGLready then begin
    glClearColor(0,0,0,0);
    glClear(GL_COLOR_BUFFER_BIT OR GL_DEPTH_BUFFER_BIT);

    Case Primitive of

    Triangle:
    begin
      glBegin(GL_TRIANGLES);
        glColor3f(1,0,0);
        glVertex3f(0,1,0);
        glColor3f(0,1,0);
        glVertex3f(-1,-1,0);
        glColor3f(0,0,1);
        glVertex3f(1,-1,0);
      glEnd;
    end;

    Square:
    begin
      glBegin(GL_TRIANGLE_STRIP);
        glColor3f(1,0,0);
        glVertex(-1,1,0);
        glColor3f(0,1,1);
        glVertex(1,1,0);
        glColor3f(1,1,0);
        glVertex(-1,-1,0);
        glColor3f(0,0,1);
        glVertex(1,-1,0);
      glEnd;
    end;

    Cube:
    begin
      glBegin(GL_POLYGON);
        glColor3f(0, 0, 1);
        glVertex3f(1.0, 1.0, 1.0);
        glVertex3f(-1.0, 1.0, 1.0);
        glVertex3f(-1.0, -1.0, 1.0);
        glVertex3f(1.0, -1.0, 1.0);
      glEnd;

      glBegin(GL_POLYGON);
        glColor3f(1, 1, 0);
        glVertex3f(1.0, 1.0, -1.0);
        glVertex3f(1.0, -1.0, -1.0);
        glVertex3f(-1.0, -1.0, -1.0);
        glVertex3f(-1.0, 1.0, -1.0);
      glEnd;

      glBegin(GL_POLYGON);
        glColor3f(1, 0, 1);
        glVertex3f(-1.0, 1.0, 1.0);
        glVertex3f(-1.0, 1.0, -1.0);
        glVertex3f(-1.0, -1.0, -1.0);
        glVertex3f(-1.0, -1.0, 1.0);
      glEnd;

      glBegin(GL_POLYGON);
        glColor3f(0, 1, 1);
        glVertex3f(1.0, 1.0, 1.0);
        glVertex3f(1.0, -1.0, 1.0);
        glVertex3f(1.0, -1.0, -1.0);
        glVertex3f(1.0, 1.0, -1.0);
      glEnd;

      glBegin(GL_POLYGON);
        glColor3f(0, 1, 0);
        glVertex3f(-1.0, 1.0, -1.0);
        glVertex3f(-1.0, 1.0, 1.0);
        glVertex3f(1.0, 1.0, 1.0);
        glVertex3f(1.0, 1.0, -1.0);
      glEnd;

      glBegin(GL_POLYGON);
        glColor3f(1, 0, 0);
        glVertex3f(-1.0, -1.0, -1.0);
        glVertex3f(1.0, -1.0, -1.0);
        glVertex3f(1.0, -1.0, 1.0);
        glVertex3f(-1.0, -1.0, 1.0);
      glEnd;
    end;

    end; {case}

    //die aktuelle Matrix speichern
    glGetFloatv(GL_MODELVIEW_MATRIX, @Matrix);
    //und ausgeben
    printMatrix;

    glFlush;
    SwapBuffers(gldc);
  end else oglScreen.Caption := 'OpenGL is not initialized!'
end;

procedure TfrmMain.btnUndoClick(Sender: TObject);
begin
  //die aktuelle Matrix ausgeben
  printMatrix;
end;

procedure TfrmMain.FormCreate(Sender: TObject);
begin
  openGLReady:= false;
  DecimalSeparator := '.';
  Primitive := Triangle;
  oglScreen.Caption := 'OpenGL is not initialized!'
end;

procedure TfrmMain.FormClose(Sender: TObject; var Action: TCloseAction);
begin
  Destroy1Click(self);
end;

procedure TfrmMain.About1Click(Sender: TObject);
begin
   Application.MessageBox(
        'Matrix Control v0.2'+#13+#13+
        'coded by Lithander in 2002'+#13+#13+
        'lithander@gmx.de'+#13+
        'http://pixelpracht.flipcode.com'+#13#13+
        'For further info and help check the Readme!',
        'About Matrix Control',
        MB_OK)
end;

procedure TfrmMain.btnApplyMatrixClick(Sender: TObject);
begin
 If OpenGLready then begin
 Matrix[0] := strtofloat(edma00.text);
 Matrix[1] := strtofloat(edma10.text);
 Matrix[2] := strtofloat(edma20.text);
 Matrix[3] := strtofloat(edma30.text);
 Matrix[4] := strtofloat(edma01.text);
 Matrix[5] := strtofloat(edma11.text);
 Matrix[6] := strtofloat(edma21.text);
 Matrix[7] := strtofloat(edma31.text);
 Matrix[8] := strtofloat(edma02.text);
 Matrix[9] := strtofloat(edma12.text);
 Matrix[10] := strtofloat(edma22.text);
 Matrix[11] := strtofloat(edma32.text);
 Matrix[12] := strtofloat(edma03.text);
 Matrix[13] := strtofloat(edma13.text);
 Matrix[14] := strtofloat(edma23.text);
 Matrix[15] := strtofloat(edma33.text);
 //Load Matrix into OpenGL
 ProtocolClearAbove;
 cbMatrices.Items.Add('Manual Change');
 cbMatrices.ItemIndex := MatrixProtocol.Index;
 MatrixProtocol.add(Matrix);
 glLoadMatrixf(@Matrix);
 Render1Click(self);
 end;
end;

procedure TfrmMain.btnTranslateClick(Sender: TObject);
var pa,pb,pc,pd : single;
    valid : boolean;
begin
  If not OpenGLready then exit;
  //getting the parameter for ogl-command
  valid := true;
  valid := valid and getParam(cbParam.text,0,pa);
  valid := valid and getParam(cbParam.text,1,pb);
  valid := valid and getParam(cbParam.text,2,pc);
  if not valid then begin
    ParamError(PE_TRANSLATE);
    exit;
  end;
  //call ogl-command
  glTranslatef(pa,pb,pc);
  Render1Click(self);
  //update protocoll
  ProtocolClearAbove;
  cbMatrices.Items.Add('glTranslate('+cbParam.text+')');
  cbMatrices.ItemIndex := MatrixProtocol.Index;
  MatrixProtocol.add(Matrix);
end;

procedure TfrmMain.btnLoadIdentityClick(Sender: TObject);
begin
  If not OpenGLready then exit;
  glLoadIdentity;
  Render1Click(self);
  cbMatrices.Items.Add('glLoadIdentity');
  cbMatrices.ItemIndex := MatrixProtocol.Index;
  MatrixProtocol.add(Matrix);
end;

procedure TfrmMain.btnScaleClick(Sender: TObject);
var pa,pb,pc : single;
    valid : boolean;
begin
  If not OpenGLready then exit;
  //getting the parameter for ogl-command
  valid := true;
  valid := valid and getParam(cbParam.text,0,pa);
  valid := valid and getParam(cbParam.text,1,pb);
  valid := valid and getParam(cbParam.text,2,pc);
  if not valid then begin
    ParamError(PE_SCALE);
    exit;
  end;
  //call ogl-command
  glScalef(pa,pb,pc);
  Render1Click(self);
  //update protocoll
  ProtocolClearAbove;
  cbMatrices.Items.Add('glScale('+cbParam.text+')');
  cbMatrices.ItemIndex := MatrixProtocol.Index;
  MatrixProtocol.add(Matrix);
end;

procedure TfrmMain.btnRotateClick(Sender: TObject);
var pa,pb,pc,pd : single;
    valid : boolean;
begin
  If not OpenGLready then exit;
  //getting the parameter for ogl-command
  valid := true;
  valid := valid and getParam(cbParam.text,0,pa);
  valid := valid and getParam(cbParam.text,1,pb);
  valid := valid and getParam(cbParam.text,2,pc);
  valid := valid and getParam(cbParam.text,3,pd);
  if not valid then begin
    ParamError(PE_ROTATE);
    exit;
  end;
  //call ogl-command
  glRotatef(pa,pb,pc,pd);
  Render1Click(self);
  //update protocoll
  ProtocolClearAbove;
  cbMatrices.Items.Add('Rotate '+cbParam.text);
  cbMatrices.ItemIndex := MatrixProtocol.Index;
  MatrixProtocol.add(Matrix);
end;

procedure TfrmMain.Triangle1Click(Sender: TObject);
begin
  Triangle1.Checked := true;
  Primitive := Triangle;
  Render1Click(self);
end;

procedure TfrmMain.Cube1Click(Sender: TObject);
begin
  Cube1.Checked := true;
  Primitive := Cube;
  Render1Click(self);
end;

procedure TfrmMain.Square1Click(Sender: TObject);
begin
  Square1.Checked := true;
  Primitive := Square;
  Render1Click(self);
end;

procedure TfrmMain.btnClearProtocolClick(Sender: TObject);
begin
  If not OpenGLready then exit;
  MatrixProtocol.Clear;
  cbMatrices.Clear;
end;

procedure TfrmMain.cbMatricesChange(Sender: TObject);
begin
  If not OpenGLready then exit;
  MatrixProtocol.activate(cbMatrices.ItemIndex);
  Render1Click(self);
end;

procedure TfrmMain.btnBackClick(Sender: TObject);
begin
  If not OpenGLready then exit;
  if cbMatrices.ItemIndex > 0 then cbMatrices.ItemIndex:=cbMatrices.ItemIndex-1;
  MatrixProtocol.activate(cbMatrices.ItemIndex);
  Render1Click(self);
end;

procedure TfrmMain.btnForwardClick(Sender: TObject);
begin
  If not OpenGLready then exit;
  if cbMatrices.ItemIndex < cbMatrices.Items.Count then cbMatrices.ItemIndex:=cbMatrices.ItemIndex+1;
  MatrixProtocol.activate(cbMatrices.ItemIndex);
  Render1Click(self);
end;

end.
